@@ -11,6 +11,7 @@ import TokenDef
 import Data.List (intersperse)
 import Data.Char
 import Scanner (Input(..),scanLit,input)
+import System.FilePath
 import Data.List
 import Expression
 import Macro --marcos
@@ -62,28 +63,34 @@ parseAGI opts searchPath file
 
 depsAG :: Options -> [FilePath] -> String -> IO ([String], [Message Token Pos])
 depsAG opts searchPath file
-  = do (_,_,fs,_,mesgs) <- parseFile False opts searchPath file
-       return (tail fs, mesgs) -- first file is always the file itself
+  = do let fn = normalise file
+       (_,_,fs,_,mesgs) <- parseFile False opts searchPath fn
+       return (filter (/= fn) fs, mesgs)
 
 -- marcos: added the parameter 'agi' and the 'ext' part
 parseFile :: Bool -> Options -> [FilePath] -> String -> IO  ([Elem],[String],[String], Maybe String,[Message Token Pos ])
-parseFile agi opts searchPath filename
- = do file <- resolveFile opts searchPath filename
-      txt <- readFile file
-      let searchPath' = takeDirectory file : searchPath  -- search first relative to the including file
-          litMode = ".lag" `isSuffixOf` file
-          (files,text) = if litMode then scanLit txt
-                         else ([],txt)
-          tokens       = input opts (initPos file) text
+parseFile = parseFile' []
 
-          steps = parse (pElemsFiles agi) tokens
-          stop (_,fs,_,_,_) = null fs
-          cont (es,fs,allfs,ext,msg)
-            = do res <- mapM (parseFile agi opts searchPath') fs
-                 let (ess,fss,allfss,_, msgs) = unzip5 res
-                 return (concat ess ++ es, concat fss, allfs ++ concat allfss, ext, msg ++ concat msgs)
-      let (Pair (es,fls,ext) _ ,mesg) = evalStepsMessages steps
-      loopp stop cont (es,files ++ fls,[file], ext,mesg)
+parseFile' :: [String] -> Bool -> Options -> [FilePath] -> String -> IO  ([Elem],[String],[String], Maybe String,[Message Token Pos ])
+parseFile' parsedfiles agi opts searchPath filename
+ = do file <- normalise `fmap` resolveFile opts searchPath filename
+      if file `elem` parsedfiles 
+        then return ([], [], parsedfiles, Nothing, [])
+        else do
+        txt <- readFile file
+        let searchPath' = takeDirectory file : searchPath  -- search first relative to the including file
+            litMode = ".lag" `isSuffixOf` file
+            (files,text) = if litMode then scanLit txt
+                           else ([],txt)
+            tokens       = input opts (initPos file) text
+
+            steps = parse (pElemsFiles agi) tokens
+            stop (_,fs,_,_,_) = null fs
+            cont (es,f:fs,allfs,ext,msg)
+              = do (ess,fss,allfss,_, msgs) <- parseFile' allfs agi opts searchPath' f
+                   return (ess ++ es, fss ++ fs, allfss, ext, msg ++ msgs)
+        let (Pair (es,fls,ext) _ ,mesg) = evalStepsMessages steps
+        loopp stop cont (es,files ++ fls,file : parsedfiles, ext,mesg)
  where
 
     --
@@ -100,10 +107,11 @@ parseFile agi opts searchPath filename
              addElem e      (es,fs,ext) = (e:es,   fs, ext)
              addInc  (fn,_) (es,fs,ext) 
                | noIncludes opts = (es, fs, ext)  -- skip includes
-               | otherwise       = (es,fn:fs, ext)
+               | otherwise       = (es,normalise fn:fs, ext)
              addExt  (fn,_) (es,fs,ext)
                | noIncludes opts = (es, fs, ext)  -- skip includes
-               | otherwise       = if agi then (es,fs, Just fn) else (es,fn:fs, ext) --marcos
+               | otherwise       = if agi then (es,fs, Just fn') else (es,fn':fs, ext) --marcos
+               where fn' = normalise fn
 
     pCodescrapL = (\(ValToken _ str pos) -> (str, pos))<$>
                         parseScrapL <?> "a code block"
@@ -150,7 +158,10 @@ parseFile agi opts searchPath filename
 
     pElem :: AGParser Elem
     pElem
-         =  Data <$> (pDATA <|> pTYPE)
+         =  (\(pos, ct) -> Data pos ct)
+                 <$> (    (\x -> (x, DataConstructor))   <$> (pDATA <|> pTYPE)
+                     <|>  (\x -> (x, RecordConstructor)) <$> pRECORD
+                     )
                  <*> pOptClassContext'
                  <*> pNontSet
                  <*> pTyVars
@@ -311,7 +322,7 @@ parseFile agi opts searchPath filename
     pSemDefs =  concat <$> pList_ng pSemDef <?> "attribute rules"
 
     pExpr :: AGParser Expression
-    pExpr = (\(str,pos) ->  Expression pos (lexTokens pos str)) <$> pCodescrapL <?> "an expression"
+    pExpr = (\(str,pos) ->  Expression pos (lexTokens opts pos str)) <$> pCodescrapL <?> "an expression"
 
     pTypeColon :: AGParser Pos
     pTypeColon
@@ -537,7 +548,7 @@ pCodescrap' = fst <$> pCodescrap
 pCodescrap ::  AGParser (String,Pos)
 pCodescrap   = pCodeBlock
 
-pSEM, pATTR, pDATA, pUSE, pLOC,pINCLUDE, pTYPE, pEquals, pColonEquals, pTilde,
+pSEM, pATTR, pDATA, pRECORD, pUSE, pLOC,pINCLUDE, pTYPE, pEquals, pColonEquals, pTilde,
       pEXTENDS, --marcos
       pBar, pColon, pLHS,pINST,pSET,pDERIVING,pMinus,pIntersect,pDoubleArrow,pArrow,
       pDot, pUScore, pEXT,pAt,pStar, pSmaller, pWRAPPER, pNOCATAS, pPRAGMA, pMAYBE, pEITHER, pMAP, pINTMAP,
@@ -552,6 +563,7 @@ pPRAGMA      = pCostReserved 90 "PRAGMA"  <?> "PRAGMA"
 pSEMPRAGMA   = pCostReserved 90 "SEMPRAGMA" <?> "SEMPRAGMA"
 pATTACH      = pCostReserved 90 "ATTACH"  <?> "ATTACH"
 pDATA        = pCostReserved 90 "DATA"    <?> "DATA"
+pRECORD      = pCostReserved 90 "RECORD"  <?> "RECORD"
 pEXT         = pCostReserved 90 "EXT"     <?> "EXT"
 pATTR        = pCostReserved 90 "ATTR"    <?> "ATTR"
 pSEM         = pCostReserved 90 "SEM"     <?> "SEM"
